
set echo on

exec DBMS_MONITOR.SESSION_TRACE_ENABLE;

DECLARE

   r_enqueue_options DBMS_AQ.ENQUEUE_OPTIONS_T;
   r_msg_properties  DBMS_AQ.MESSAGE_PROPERTIES_T;
   nt_msg_properties DBMS_AQ.MESSAGE_PROPERTIES_ARRAY_T;
   nt_payloads       demo_aq_array_ntt;
   nt_msg_ids        DBMS_AQ.MSGID_ARRAY_T;
   v_enqueued_cnt    PLS_INTEGER;

BEGIN

   /* Initialise the collections... */
   nt_payloads := demo_aq_array_ntt();
   nt_msg_properties := DBMS_AQ.MESSAGE_PROPERTIES_ARRAY_T();
   nt_msg_ids := DBMS_AQ.MSGID_ARRAY_T();

   /* Load up 2500 arbitrary messages... */
   FOR i IN 1 .. 2500 LOOP

      nt_payloads.EXTEND;
      nt_payloads(nt_payloads.LAST) := demo_aq_array_ot(
                                          'Message number ' || TO_CHAR(i)
                                          );

      nt_msg_properties.EXTEND;
      nt_msg_properties(nt_msg_properties.LAST) := r_msg_properties;

   END LOOP;

   /* Enqueue the messages in a single transaction... */
   v_enqueued_cnt := DBMS_AQ.ENQUEUE_ARRAY(
                        queue_name               => 'demo_aq_array_q',
                        enqueue_options          => r_enqueue_options,
                        array_size               => nt_payloads.COUNT,
                        message_properties_array => nt_msg_properties,
                        payload_array            => nt_payloads,
                        msgid_array              => nt_msg_ids
                        );
   COMMIT;

END;
/


DECLARE

   r_dequeue_options DBMS_AQ.DEQUEUE_OPTIONS_T;
   nt_msg_properties DBMS_AQ.MESSAGE_PROPERTIES_ARRAY_T;
   nt_payloads       demo_aq_array_ntt;
   nt_msg_ids        DBMS_AQ.MSGID_ARRAY_T;
   v_dequeued_cnt    PLS_INTEGER;
   v_dequeue_batch   PLS_INTEGER := 500;
   v_continue        BOOLEAN := TRUE;

   x_timeout EXCEPTION;
   PRAGMA EXCEPTION_INIT(x_timeout, -25228);

BEGIN

   /* Prepare collections... */
   nt_payloads := demo_aq_array_ntt();
   nt_payloads.EXTEND(v_dequeue_batch);
   nt_msg_properties := DBMS_AQ.MESSAGE_PROPERTIES_ARRAY_T();
   nt_msg_properties.EXTEND(v_dequeue_batch);
   nt_msg_ids := DBMS_AQ.MSGID_ARRAY_T();

   /* Set a 5 second timeout for the dequeue... */
   r_dequeue_options.wait := 5;

   /* Dequeue all messages in batches... */
   WHILE v_continue LOOP

      BEGIN

         v_dequeued_cnt := DBMS_AQ.DEQUEUE_ARRAY(
                              queue_name               => 'demo_aq_array_q',
                              dequeue_options          => r_dequeue_options,
                              array_size               => v_dequeue_batch,
                              message_properties_array => nt_msg_properties,
                              payload_array            => nt_payloads,
                              msgid_array              => nt_msg_ids
                              );

         /* Handle exit scenario one from the notes above... */
         v_continue := (v_dequeued_cnt = v_dequeue_batch);

         /* Commit the batch... */
         COMMIT;

      EXCEPTION

         /* Handle exit scenario two from the notes above... */
         WHEN x_timeout THEN
            v_continue := FALSE;

      END;

   END LOOP;

END;
/

exec DBMS_MONITOR.SESSION_TRACE_DISABLE;
